#!/usr/bin/python
# -*- coding: utf-8 -*-

# Screen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.HelpMenu import HelpableScreen

# GUI (Summary)
from Screens.Setup import SetupSummary

from Components.Label import Label
from Components.ScrollLabel import ScrollLabel
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap, HelpableActionMap

# GUI (System) 
from enigma import eTimer, ePoint, eSize, getDesktop, ePoint

# Configuration
from Components.config import *
from Components.ConfigList import ConfigListScreen

from Tools.Notifications import AddPopup
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS

import Screens.Standby
import os, sys, traceback
import subprocess, threading
import time
import datetime
#import UpdateCheck_Setup

PluginVersion = ""
reload_value = True

sz_w = getDesktop(0).size().width()

class UpdateCheckSetup(Screen, ConfigListScreen):
	
	if sz_w == 1920:
		skin = """
		<screen name="UpdateCheckSetup" position="center,center" size="1200,820">
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="295,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="305,5" size="295,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="600,5" size="295,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="895,5" size="295,70" scale="stretch" alphatest="on" />
			<widget source="key_red" render="Label" position="10,5" zPosition="1" size="295,70" font="Regular;30" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="key_green" render="Label" position="310,5" zPosition="1" size="300,70" font="Regular;30" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
			<widget source="key_yellow" render="Label" position="610,5" zPosition="1" size="300,70" font="Regular;30" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
			<widget source="key_blue" render="Label" position="910,5" zPosition="1" size="300,70" font="Regular;30" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
			<widget name="config" position="10,90" itemHeight="35" size="1180,540" enableWrapAround="1" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="10,650" zPosition="2" size="1180,2" />
			<widget name="help" position="10,655" size="1180,145" font="Regular;32" />
		</screen>"""
	else:
		skin = """
		<screen name="UpdateCheckSetup" position="center,center" size="800,530">
			<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="200,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="400,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="600,0" size="200,40" scale="stretch" alphatest="on" />
			<widget source="key_red" render="Label" position="0,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="key_green" render="Label" position="200,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
			<widget source="key_yellow" render="Label" position="400,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
			<widget source="key_blue" render="Label" position="600,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
			<widget name="config" position="5,50" itemHeight="30" size="790,380" enableWrapAround="1" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,431" zPosition="2" size="800,2" />
			<widget name="help" position="5,435" size="790,75" font="Regular;22" />
		</screen>"""


	def __init__(self, session):
		Screen.__init__(self, session)

		# Summary
		global PluginVersion
		from plugin import VERSION
		PluginVersion = VERSION
		self.skinName = "UpdateCheckSetup"
		self.setup_title = _("UpdateCheck Setup ") + str(PluginVersion)
		self.onChangedEntry = []
		
		self.list = []
		self.buildConfig()
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changed)
		
		def selectionChanged():
			if self["config"].current:
				self["config"].current[1].onDeselect(self.session)
			self["config"].current = self["config"].getCurrent()
			if self["config"].current:
				self["config"].current[1].onSelect(self.session)
			for x in self["config"].onSelectionChanged:
				x()
		self["config"].selectionChanged = selectionChanged
		self["config"].onSelectionChanged.append(self.updateHelp)

		# Initialize widgets
		self["key_green"] = StaticText(_("OK"))
		self["key_red"] = StaticText(_("Cancel"))
		self["key_yellow"] = StaticText("")
		#self["key_yellow"] = StaticText("Notification")
		self["key_blue"] = StaticText("Test")
		self["help"] = Label("")
		
		# Define Actions
		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
			{
				"cancel": 	self.keyCancel,
				"save": 		self.keySave,
				"blue":			self.test,
				#"yellow":		self.notify,
				"ok": 			self.ok,
			},-2)

		# Trigger change
		self.changed()

		self.onLayoutFinish.append(self.layoutFinished)

	def notify(self):
		from Screens.NotificationQueueViewer import NotificationQueueViewer
		self.session.open(NotificationQueueViewer)
		

	def buildConfig(self):
		
		self.list.append( getConfigListEntry(_("Updatecheck und Info beim Start der Box"), config.plugins.updatecheck.enable_autocheck, _("Prüft beim Box-Start (auch GUI-Neustart und Start aus dem Standby) auf Updates und gibt eine Meldung aus (nein/nur bei Updates/immer). Bei 'immer' kommt auch eine Info-Meldung, dass kein Update verfügbar ist.")) )
		
		if config.plugins.updatecheck.enable_autocheck.value != "False":
			self.list.append( getConfigListEntry(_("   Update-Verzögerung nach Box-Start (in Sek)"), config.plugins.updatecheck.updatecheck_delay, _("Legt fest, mit wieviel Sekunden Verzögerung der Updatecheck nach Boxstart ausgeführt wird.")) )
			
			self.list.append( getConfigListEntry(_("   UpdateCheck-Timeout (in Sek)"), config.plugins.updatecheck.update_timeout, _("Legt fest, nach wieviel Sekunden die Update-Prozesse z.B. bei Update-Server-Problemen spätestens beendet werden sollen (5-20, default=5)")) )
			
			self.list.append( getConfigListEntry(_("   Anzeigedauer für Update-Info-Fenster (in Sek)"), config.plugins.updatecheck.popups_timeout, _("Legt fest, nach wieviel Sekunden das Update-Info-Fenster automatisch geschlossen wird (1-20 Sek., 0 = wartet auf Tastendruck)")) )
			
			self.list.append( getConfigListEntry(_("   Ingnoriere hold-Pakete bei der Update-Info"), config.plugins.updatecheck.ignore_hold, _("Pakete, welche auf 'hold' gesetzt wurden, werden bei 'Ja' bei der Update-Prüfung nicht gemeldet.")) )
			
			self.list.append( getConfigListEntry(_("   erweiterte Update-Info mit Frage zum Öffnen des SoftwareManagers"), config.plugins.updatecheck.show_updatequestion, _("Zeigt beim Update-Hinweis zusätzlich eine Frage, ob man den SoftwareManager direkt öffnen möchte.")) )
			
			if config.plugins.updatecheck.show_updatequestion.value == "True":
				self.list.append( getConfigListEntry(_("       Antwort für Frage steht standardmäßig auf:"), config.plugins.updatecheck.updatequestion_defaultanswer, _("Legt fest, welche Antwort für die Frage zum Öffnen des SoftwareManagers standardmäßig vorausgewählt sein soll. Bei 'nein' öffnet sich der SoftwareManager bei Zeitablauf der Frage nicht automatisch.")) )


	def layoutFinished(self):
		self.setTitle(self.setup_title)

	def ok(self):
		pass
		#if self["config"].getCurrent()[1] == config.plugins.updatecheck. ...:
		#	pass

	def setCustomTitle(self):
		self.setTitle(self.setup_title)

	def updateHelp(self):
		cur = self["config"].getCurrent()
		if cur:
			self["help"].text = cur[2]

	def changeConfig(self):
		self.list = []
		self.buildConfig()
		self["config"].setList(self.list)

	def changed(self):
		for x in self.onChangedEntry:
			x()
		current = self["config"].getCurrent()[1]
		if (current == config.plugins.updatecheck.enable_autocheck) or (current == config.plugins.updatecheck.show_updatequestion):
			self.changeConfig()
			return

	def getCurrentEntry(self):
		return self["config"].getCurrent()[0]

	def getCurrentValue(self):
		return str(self["config"].getCurrent()[1].getText())

	def createSummary(self):
		return SetupSummary

	def test(self):
		self.session.open(UpdateScreen)

########################################################################### 

class Command(object):
	def __init__(self, cmd):
		self.cmd = cmd
		self.process = None

	def run(self, timeout):
		def target():
			print '=====[UpdateCheck] command-Thread started'
			self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			(self.out,self.err) = self.process.communicate()
			print '=====[UpdateCheck] command-Thread finished'

		thread = threading.Thread(target=target)
		thread.start()
		self.timeout=False

		thread.join(float(timeout))
		if thread.is_alive():
			print '=====[UpdateCheck] Terminating command-process by timeout'
			self.process.terminate()
			self.timeout=True
			thread.join()

class UpdateScreen(Screen, HelpableScreen):
	
	if sz_w == 1920:
		skin = """
		<screen name="UpdateCheck" position="center,center" size="1200,680">
			<widget name="myLabel" position="15,10" size="650,80" font="Regular;24"/>
			<widget name="status" position="15,150" size="650,160" font="Regular;32"/>
			<widget name="updates" position="15,350" size="1150,260" font="Regular;20"/>
			
			<widget source="global.CurrentTime" render="Label" position="700,15" size="475,40" font="Regular;32" halign="right">
				<convert type="ClockToText">Format:%A %d.%m.  %H:%M</convert>
			</widget>

			<eLabel text="Menü" position="1080,60" size="90,30" backgroundColor="#777777" halign="center" font="Regular;24"/>

		</screen>"""
	else:
		skin = """
		<screen name="UpdateCheck" position="center,center" size="930,520">
			<widget name="myLabel" position="15,20" size="500,50" font="Regular;18"/>
			<widget name="status" position="15,100" size="750,120" font="Regular;24"/>
			<widget name="updates" position="15,250" size="900,260" font="Regular;16"/>
			
			<widget source="global.CurrentTime" render="Label" position="540,20" size="380,30" font="Regular;24" halign="right">
				<convert type="ClockToText">Format:%A %d.%m.  %H:%M</convert>
			</widget>
			
			<eLabel text="Menü" position="850,60" size="70,20" backgroundColor="#777777" halign="center" font="Regular;18"/>

		</screen>"""
	
	def __init__(self, session, args = 0):
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		try:
			self["myLabel"] = Label("")
			self["status"] = Label("")
			self["updates"] = ScrollLabel("")
			global PluginVersion
			from plugin import VERSION
			PluginVersion = VERSION
			self.skinName = "UpdateCheck"
			
			self["myActionMap"] = ActionMap(["InfobarEPGActions", "OkCancelActions", "MovieSelectionActions", "DirectionActions"],
				{
						"cancel"			 : self.cancel,
						"showEventInfo"	 : self.checkUpdate,
						"contextMenu"  	 : self.setup,
						"ok"				 : self.ok,
						"up":    self["updates"].pageUp,
						"down":  self["updates"].pageDown,
				},-1)
			
			# add helpstrings to helpscreen
			self.helpList.append((self["myActionMap"], "OkCancelActions", [("cancel", _("Plugin beenden"))]))
			self.helpList.append((self["myActionMap"], "MovieSelectionActions", [("contextMenu", _("Setup öffnen"))]))
			self.helpList.append((self["myActionMap"], "InfobarEPGActions", [("showEventInfo", _("Updateprüfung"))]))

			self.onLayoutFinish.append( self.LayoutFinish )

		except:
			import traceback
			traceback.print_exc()

	def LayoutFinish(self):
			self.setTitle("UpdateCheck - " + PluginVersion)
			self["myLabel"].setText(_("Zum Prüfen die OK-Taste drücken."))
			self["status"].setText(_("Update-Status..."))
			self["updates"].setText(_("Updates..."))

	def setup(self):
			
			self.session.open(UpdateCheckSetup)

	def ok(self):
		self["status"].setText(_("Prüfe Update-Status. Bitte warten ..."))
		self.updateTimer = eTimer()
		self.updateTimer_conn = self.updateTimer.timeout.connect(self.checkUpdate)
		self.updateTimer.start(500)

	def checkUpdate(self):
			
			self.updateTimer = None
			self.updateTimer_conn = None
			
			command = Command("apt-get update")
			command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
			
			if command.timeout:
				self["status"].setText("update timeout")
				return
			elif command.process.returncode == 0:
				self["status"].setText("update complete" + str(command.out))
			else:
				self["status"].setText("update error: " + str(command.process.returncode) + " - " + str(command.err))
				return
			
			command = Command("apt list --upgradable")
			command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
			
			if command.timeout:
				self["status"].setText("list --upgradable timeout")
				return
			elif command.process.returncode != 0:
				self["status"].setText("list --upgradable error: " + str(command.process.returncode) + " - " + str(command.err))
				return
			
			check_update_txt = command.out
			
			updates = check_update_txt.split("\n")
			all_updates = list(updates)
			print "== check_update:", check_update_txt
			print "== update:", updates, len(updates)
			
			hold_packages = []
			
			try:
				if config.plugins.updatecheck.ignore_hold.value:
					print "== check hold_packages"
					hold_packages_txt = subprocess.check_output("cat /var/lib/dpkg/status | grep -B5 hold | grep Package", shell=True)

					#prepare hold-list
					if len(hold_packages_txt):
						packages = hold_packages_txt.split("\n")
						for package in packages:
							if "Package: " in package:
								hold_packages.append(package.replace("Package: ","") + "/")
					
					#clean updates-list by hold-list
					for hold_package in hold_packages:
						index = [i for i,x in enumerate(updates) if hold_package in x]
						if len(index):
							print "== index:", index, updates[index[0]]
							del updates[index[0]]
				
			except:
				import traceback
				traceback.print_exc()
			
			print "== hold_packages:", hold_packages
			
			if len(updates)==3:
				self["status"].setText("Es liegt 1 Feed-Update vor!")
			elif len(updates)>3:
				self["status"].setText("Es liegen " + str(len(updates)-2) + " Feed-Updates vor!")
			else:
				if all_updates != updates:
					self["status"].setText("keine Feed-Updates gefunden! Nur hold-Updates.")
				else:
					self["status"].setText("keine Feed-Updates gefunden!")
			
			try:
				self["updates"].setText("Updates:\n" + "\n".join(all_updates[1:]) + "\nhold:\n" + "\n".join(hold_packages))
			except:
				import traceback
				traceback.print_exc()

	def cancel(self):
			self.close()


#=========== for autoupdatecheck ================================

gl_udc_session = None

def updatecheck(udc_session):
			
			import Screens.Standby
			if Screens.Standby.inStandby:
				print "=====[UpdateCheck] inStandby - don't check for updates"
				return
			
			global gl_udc_session
			gl_udc_session = udc_session
			
			startUpdateCheck()

def startUpdateCheck():
			
			global gl_udc_session
			session = gl_udc_session
			
			command = Command("apt-get update")
			command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
			
			if command.timeout:
				AddPopup(_("\n = UpdateCheck = \n\n'update' timed out!!!"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
				return
			elif command.process.returncode != 0:
				AddPopup("\n = UpdateCheck = \n\n'update' - "+ str(command.process.returncode) + " - " + str(command.err), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
				return
			
			command = Command("apt list --upgradable")
			command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
			
			if command.timeout:
				AddPopup(_("\n = UpdateCheck = \n\n'list --upgradable' timed out!!!"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
				return
			elif command.process.returncode != 0:
				AddPopup("\n = UpdateCheck = \n\n'list --upgradable' - "+ str(command.process.returncode) + " - " + str(command.err), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
				return
			
			check_update_txt = command.out
			
			updates = check_update_txt.split("\n")
			print "== check_update:", check_update_txt
			print "== update:", updates, len(updates)
			
			UpdateExist = False
			
			hold_packages = []
			
			try:
				if config.plugins.updatecheck.ignore_hold.value:
					#read hold-packages
					hold_packages_txt = ""
					try:
						hold_packages_txt = subprocess.check_output("cat /var/lib/dpkg/status | grep -B5 hold | grep Package", shell=True)
					except:
						print "=====[UpdateCheck] no hold packages found"
						pass

					#prepare hold-list
					if len(hold_packages_txt):
						packages = hold_packages_txt.split("\n")
						for package in packages:
							if "Package: " in package:
								hold_packages.append(package.replace("Package: ","") + "/")
					
					#clean updates-list by hold-list
					for hold_package in hold_packages:
						index = [i for i,x in enumerate(updates) if hold_package in x]
						if len(index):
							print "== index:", index, updates[index[0]]
							del updates[index[0]]
				
			except:
				import traceback
				traceback.print_exc()
			
			if len(updates)==3:
				check_update_txt = _("Es liegt 1 Feed-Update vor!") + "\n - " + str(updates[1].split("/")[0])
				print "=====[UpdateCheck] 1 update found"
				UpdateExist = True
			elif len(updates)>3:
				check_update_txt = _("Es liegen %s Feed-Updates vor!") % str(len(updates)-2)
				check_update_txt += "\n - " + str(updates[1].split("/")[0]) + "\n - " + str(updates[2].split("/")[0])
				if len(updates)>4:
					check_update_txt += "\n - " + str(updates[3].split("/")[0])
				if len(updates)>5:
					check_update_txt += "\n ..."
				
				print "=====[UpdateCheck] %s updates found" % str(len(updates)-2)
				UpdateExist = True
			else:
				check_update_txt = _("keine Feed-Updates gefunden!")
				print "=====[UpdateCheck] no updates found"
			
			if UpdateExist:
				if config.plugins.updatecheck.enable_autocheck.value != "False":
					if config.plugins.updatecheck.show_updatequestion.value == "False":
						AddPopup("\n = UpdateCheck = \n\n" + check_update_txt + "\n\n\n" + _("zum Update den SoftwareManager öffnen"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
					else:
						#open only if enigma is start complete
						#from Tools import Notifications
						#Notifications.AddNotification(UpdateInfo, check_update_txt)
						session.open(UpdateInfo, check_update_txt)
			else:
				if config.plugins.updatecheck.enable_autocheck.value == "True":
					AddPopup("\n = UpdateCheck = \n\n" + check_update_txt, MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'GPU_PopUp_Update')


class UpdateInfo(Screen):
	
	skin = """ <screen position="90,90" size="0,0" title="Update Info"></screen>"""

	def __init__(self, session, UpdateText=""):
		self.session = session
		Screen.__init__(self, session)
		
		try:
			self.UpdateText = UpdateText
			self.firsttime = True
			self.onShown.append(self.showmsg)
			self.onLayoutFinish.append( self.LayoutFinish )
			
		except:
			import traceback
			traceback.print_exc()

	def LayoutFinish(self):
		
		self.setTitle("UpdateCheck - Updateinfo")

	def showmsg(self):
		
		if self.firsttime:
			self.firsttime = False
			default_answer = False
			if config.plugins.updatecheck.updatequestion_defaultanswer.value == "True":
				default_answer = True
			self.session.openWithCallback(self.openUDC, MessageBox, "\n = UpdateCheck = \n\n" + self.UpdateText +"\n\n\nSoll der SoftwareManager geöffnet werden?", MessageBox.TYPE_YESNO, timeout = int(config.plugins.updatecheck.popups_timeout.value), default = default_answer)
	
	def openUDC(self, confirm):
		
		if confirm:
			#check to open GP3/GP4 AddonManager
			if fileExists(resolveFilename(SCOPE_PLUGINS, "GP4/geminiaddonmanager/gaddon.pyo")):
				print "== import GP4 gAddons"
				from Plugins.GP4.geminiaddonmanager.gaddon import gAddonScreen
				self.session.open(gAddonScreen)
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "Bp/geminimain/gAddons.pyo")):
				print "== import GP3 gAddons"
				from Plugins.Bp.geminimain.gAddons import gAddons
				self.session.open(gAddons)
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "SystemPlugins/SoftwareManager/plugin.py")):
				from Plugins.SystemPlugins.SoftwareManager.plugin import PluginManager
				self.session.open(PluginManager)
			else:
				self.session.open(MessageBox, _("The Softwaremanagement extension is not installed!\nPlease install it."), type = MessageBox.TYPE_INFO,timeout = 10 )
		self.close()

